'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var express = _interopDefault(require('express'));
var server = require('react-dom/server');
var reactRouterDom = require('react-router-dom');
var React = _interopDefault(require('react'));

const handleRender = (App, routes = []) => (req, response, next) => {
  if (!App) {
    return response.render('index.ejs', {
      title: '',
      html: '',
      appData: {}
    });
  }

  const activeRoute = routes.find(r => reactRouterDom.matchPath(req.url, r)) || {};
  const promise = activeRoute.fetchInitialData ? activeRoute.fetchInitialData(req.url) : Promise.resolve([]);
  return promise.then(res => {
    let appData = {};

    if (res.length && Array.isArray(activeRoute.providers)) {
      appData = activeRoute.providers.reduce((acc, nextProvider, i) => {
        acc[nextProvider] = Array.isArray(res[0]) ? res[i] : res;
        return acc;
      }, appData);
    }
    const title = 'my title';
    const html = server.renderToNodeStream(React.createElement(reactRouterDom.StaticRouter, {
      location: req.url,
      context: appData
    }, React.createElement(App, {
      userAgent: req.headers['user-agent'],
      routes: routes
    })));
    const state = {
      title,
      html,
      appData
    };
    return  response.render('index.ejs', state);
  }).catch(err => {
    console.log('err', err.stack); // eslint-disable-line no-console

    return next(err);
  });
};

const render = (App, routes) => {
  const route = express.Router();
  route.get('/*', handleRender(App, routes));
  return route;
};

module.exports = render;
