import express from 'express';
import { renderToNodeStream } from 'react-dom/server';
import { matchPath, StaticRouter } from 'react-router-dom';
import React from 'react';

const handleRender = (App, routes = []) => (req, response, next) => {
  if (!App) {
    return response.render('index.ejs', {
      title: '',
      html: '',
      appData: {}
    });
  }

  const activeRoute = routes.find(r => matchPath(req.url, r)) || {};
  const promise = activeRoute.fetchInitialData ? activeRoute.fetchInitialData(req.url) : Promise.resolve([]);
  return promise.then(res => {
    let appData = {};

    if (res.length && Array.isArray(activeRoute.providers)) {
      appData = activeRoute.providers.reduce((acc, nextProvider, i) => {
        acc[nextProvider] = Array.isArray(res[0]) ? res[i] : res;
        return acc;
      }, appData);
    }
    const title = 'my title';
    const html = renderToNodeStream(React.createElement(StaticRouter, {
      location: req.url,
      context: appData
    }, React.createElement(App, {
      userAgent: req.headers['user-agent'],
      routes: routes
    })));
    const state = {
      title,
      html,
      appData
    };
    return  response.render('index.ejs', state);
  }).catch(err => {
    console.log('err', err.stack); // eslint-disable-line no-console

    return next(err);
  });
};

const render = (App, routes) => {
  const route = express.Router();
  route.get('/*', handleRender(App, routes));
  return route;
};

export default render;
